import copy
import ntpath
import os
import sys
import warnings

import numpy as np
import torch
import torch.nn as nn
import tqdm

from data import create_dataloader
from metric import create_metric_models
from metric import get_cityscapes_mIoU
from metric import get_fid
from models import create_model
from options.test_options import TestOptions
from utils import html, util


def save_images(webpage, visuals, image_path, opt):
    def convert_visuals_to_numpy(visuals):
        for key, t in visuals.items():
            tile = opt.batch_size > 8
            if key == 'labels':
                t = util.tensor2label(t, opt.input_nc + 2, tile=tile)
            else:
                t = util.tensor2im(t, tile=tile)
            visuals[key] = t
        return visuals

    visuals = convert_visuals_to_numpy(visuals)

    image_dir = webpage.get_image_dir()
    short_path = ntpath.basename(image_path[0])
    name = os.path.splitext(short_path)[0]

    webpage.add_header(name)
    ims = []
    txts = []
    links = []

    for label, image_numpy in visuals.items():
        image_name = os.path.join(label, '%s.png' % (name))
        save_path = os.path.join(image_dir, image_name)
        util.save_image(image_numpy, save_path, create_dir=True)

        ims.append(image_name)
        txts.append(label)
        links.append(image_name)
    webpage.add_images(ims, txts, links, width=opt.display_winsize)


def check(opt):
    assert opt.serial_batches
    assert opt.no_flip
    assert opt.load_size == opt.crop_size
    assert opt.preprocess == 'resize_and_crop'
    assert opt.batch_size == 1

    if not opt.no_fid:
        assert opt.real_stat_path is not None
    if opt.phase == 'train':
        warnings.warn('You are using training set for inference.')

def BIG_loss(model,size1):
    loss_nowg = 0.0
    loss_nowb = 0.0
    loss_nowwg = 0.0
    loss_nowwb = 0.0
    idx = 0
    loss = 0.0
    now_pp = 1.0
    now_qq = 0
    now_con = 0
    p = 0
    q = 0
    height = size1
    width = size1
    sparse_loss_dict = {}
    sparse_loss_dict_index = {}
    channel = 0
    model1 = model.netG.modules()
    for m in model1:
        # print(m)
        if isinstance(m, nn.InstanceNorm2d):
            p_in = m.weight
            q_in = m.bias
            idx += 1
            if p_in is None:
                # print ('None')
                now_pp = 1.0
            else:
                now_pp = p_in.abs()
                # loss_nowg = loss_nowg+now_pp.sum()
            if q_in is None:
                # print ('None')
                channel = 0
                now_qq = 0.0
            else:
                now_qq = q_in.abs()
                channel = len(q_in) 
                # loss_nowb = loss_nowb+now_qq.sum()

        if isinstance(m, nn.Conv2d) or isinstance(m, nn.ConvTranspose2d):
            p_conv = m.weight

            now_con1 = p_conv * p_conv
            now_con = p_conv.abs()
            # now_con = p_conv
            if isinstance(m, nn.Conv2d):
                now_conb = now_con[:, :, :,:].sum(axis=(2,3))
                now_conb1 = now_conb.abs()
                now_conb2 = now_conb1[:, :].sum(axis=(0))
                now_cong = now_con1[:, :, :,:].sum(axis=(2,3))+0.0000001
                now_cong1 = now_cong.sqrt()
                now_cong2 = now_cong1[:, :].sum(axis=(0))
            else:
                now_conb = now_con[:, :, :,:].sum(axis=(2,3))
                now_conb1 = now_conb.abs()
                now_conb2 = now_conb1[:, :].sum(axis=(1))
                now_cong = now_con1[:, :, :,:].sum(axis=(2,3))+0.0000001
                now_cong1 = now_cong.sqrt()
                now_cong2 = now_cong1[:, :].sum(axis=(1))
                
            t = np.sqrt(width*height)

            # loss_nowwg = loss_nowwg+now_cong2.sum()
            # loss_nowwb = loss_nowwb+now_conb2.sum()
            loss_now = now_pp* now_cong2 + now_qq * now_conb2
            # loss_now = (now_pp + now_qq) * now_con
            sparse_loss_dict[idx] = loss_now
            sparse_loss_dict_index[idx] = []
            if now_pp == 1.0:
                channel = len(now_cong2)
                for j in range(channel):
                    sparse_loss_dict_index[idx].append(2)
            else:
                for j in range(channel):
                    if q_in[j] <= -now_pp[j] * t:
                        sparse_loss_dict[idx][j] = 0
                        sparse_loss_dict_index[idx].append(0)
                    elif q_in[j] > now_pp[j] * t and now_pp[j]>0:
                        sparse_loss_dict[idx][j] = now_pp[j]* now_cong2[j]
                        sparse_loss_dict_index[idx].append(1)
                    elif now_pp[j]==0:
                        sparse_loss_dict[idx][j] = 0
                        sparse_loss_dict_index[idx].append(0)
                    else:
                        sparse_loss_dict_index[idx].append(2)
            loss = loss + sparse_loss_dict[idx].sum()
            
            now_pp = 0.0
            now_qq = 0.0
            width /= m.stride[0]
            height /= m.stride[1]
    # print(loss,loss_nowg,loss_nowb,loss_nowwg,loss_nowwb)
    return loss, sparse_loss_dict, sparse_loss_dict_index

def update_in(model, sparse_loss_dict, sparse_loss_dict_index,size1,thre):
    idx = 0
    height = size1
    width = size1
    contral_rate = 0.001
    model1 = model.netG.modules()
    crol_tem = 1
    channel_prun = []
    channel_prun_dic = {}
    cnt = 0
    for m in model1:
        if isinstance(m, nn.InstanceNorm2d):
            crol_tem = 1
            idx += 1
            if idx in sparse_loss_dict:
                gamma1 = sparse_loss_dict[idx]
                gamma_max = 0.0
                beta_max = 0.0
                gamma_bord = 0.0
                beta_bord = 0.0
                gamma1_sorted, indices = torch.sort(gamma1)
                tensor_size = len(gamma1)
                for i in range(tensor_size):
                    if sparse_loss_dict_index[idx][i]==1:
                        beta_bord = beta_bord + gamma1[i]
                        beta_max = max(beta_max,gamma1[i])
                    else:
                        gamma_bord = gamma_bord + gamma1[i]
                        gamma_max = max(gamma_max,gamma1[i])

                gamma_bord_f = gamma_bord*contral_rate
                beta_bord_f = beta_bord*contral_rate/10
                # gamma_max = gamma_max/10
                # beta_max = beta_max/20
                # gamma_max = gamma_max/8
                # beta_max = beta_max/16
                gamma_max = gamma_max/thre
                beta_max = beta_max/10
                channel_prun = []
                for i in range(tensor_size):
                    tem_sum = gamma1_sorted[:i].sum()
                    if tem_sum > beta_bord_f and tem_sum > gamma_bord_f:
                        break
                    if sparse_loss_dict_index[idx][indices[i]]==1:
                        if tem_sum <= beta_bord_f and gamma1[indices[i]] < beta_max:
                            cnt += 1
                            # print(idx,indices[i])
                            channel_prun.append(indices[i])
                            channel_prun_dic.setdefault(idx,[]).append([indices[i],gamma1[indices[i]],beta_max*10],beta_max)
                    else:
                        if tem_sum <= gamma_bord_f and gamma1[indices[i]] <= gamma_max:
                            # m.weight.data[indices[i]] = 0.0
                            # m.bias.data[indices[i]] = 0.0
                            cnt += 1
                            # print(idx,indices[i])
                            channel_prun.append(indices[i])
                            channel_prun_dic.setdefault(idx,[]).append([indices[i],gamma1[indices[i]],gamma_max*thre,gamma_bord])
                # if cnt>0:
                #     print ('layer = {}, zero cnt = {}'.format(idx, cnt))
        if crol_tem == 1 and isinstance(m, nn.Conv2d):        
            width /= m.stride[0]
            height /= m.stride[1]
            for item in channel_prun:
                m.weight.data[:, item, :,:] = 0
            crol_tem = 0 
        if crol_tem == 1 and isinstance(m, nn.ConvTranspose2d):        
            width /= m.stride[0]
            height /= m.stride[1]
            # for item in channel_prun:
            #     m.weight.data[item, :, :,:] = 0
            crol_tem = 0    
            # print ('layer = {}, zero cnt = {}'.format(idx, cnt))
    print ('layer = {}, zero cnt = {}'.format(idx, cnt))
    # for key in channel_prun_dic:
    #     print(key,channel_prun_dic[key])
    print(channel_prun_dic)
    channel_final = []
    for key in sparse_loss_dict:
        if len(sparse_loss_dict[key])>1:
            if key in channel_prun_dic:
                channel_final.append(len(sparse_loss_dict[key])-len(channel_prun_dic[key]))
                print(key,len(sparse_loss_dict[key]),len(channel_prun_dic[key]))
            else:
                channel_final.append(len(sparse_loss_dict[key]))
    print(channel_final)

if __name__ == '__main__':
    opt = TestOptions().parse(verbose=False)
    # print(' '.join(sys.argv))
    # for thre in range(15,33):
    #     print(thre)
    dataloader = create_dataloader(opt)
    model = create_model(opt)
    model.setup(opt)

    size1 = opt.crop_size
    loss_G_sparse, sparse_loss_dict, sparse_loss_dict_index = BIG_loss(model,size1)
    # file_handle1 =open('./results/big_loss.txt', mode='w+', encoding='utf8')
    # for key in sparse_loss_dict:
    #     print(key,sparse_loss_dict[key], file=file_handle1)
    # file_handle1.close()
    # print(sparse_loss_dict)
    # print('success')
    update_in(model, sparse_loss_dict, sparse_loss_dict_index, size1, 3.3)

    web_dir = opt.results_dir  # define the website directory
    webpage = html.HTML(web_dir, 'restore_G_path: %s' % (opt.restore_G_path))
    fakes, names = [], []
    for i, data in enumerate(tqdm.tqdm(dataloader)):
        model.set_input(data)  # unpack data from data loader
        if i == 0 and opt.need_profile:
            model.profile()
        model.test()  # run inference
        visuals = model.get_current_visuals()  # get image results
        generated = visuals['fake_B'].cpu()
        fakes.append(generated)
        for path in model.get_image_paths():
            short_path = ntpath.basename(path)
            name = os.path.splitext(short_path)[0]
            names.append(name)
        if i < opt.num_test:
            save_images(webpage, visuals, model.get_image_paths(), opt)
    webpage.save()  # save the HTML

    device = copy.deepcopy(model.device)
    del model
    torch.cuda.empty_cache()

    inception_model, drn_model = create_metric_models(opt, device)
    if inception_model is not None:
        npz = np.load(opt.real_stat_path)
        fid = get_fid(fakes, inception_model, npz, device, opt.batch_size)
        print('fid score: %.2f' % fid, flush=True)
    if drn_model is not None:
        mIoU = get_cityscapes_mIoU(fakes, names, drn_model, device,
                                data_dir=opt.cityscapes_path,
                                batch_size=opt.batch_size,
                                num_workers=opt.num_threads)
        print('mIoU: %.2f' % mIoU)
